#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define BLOCK_SIZE	512
#define	OFFSET_BSIZE	0x00b
#define	OFFSET_CSIZE	0x00d
#define	OFFSET_RSIZE	0x00e
#define	OFFSET_NBFAT	0x010
#define	OFFSET_COUNT16	0x013
#define OFFSET_FSIZE16	0x016
#define	OFFSET_COUNT32	0x020
#define OFFSET_FSIZE32	0x024
#define	OFFSET_FSPOS	0x030

#define OFFSET_FREE	0x1E8
#define OFFSET_ALLOC	0x1EC

int main(int argc,char *argv[]){
if(argc!=2){
  fprintf(stderr,"Usage: %s <block device>\n",argv[0]);
  exit(EXIT_FAILURE);
  }
char *device=argv[1];
int b=open(device,O_RDONLY);
if(b<0){
  fprintf(stderr,"Cannot open '%s'\n",device);
  exit(EXIT_FAILURE);
  }
unsigned char block[BLOCK_SIZE];
int nb=read(b,block,BLOCK_SIZE);
if(nb!=BLOCK_SIZE){
  fprintf(stderr,"Cannot read boot sector\n");
  exit(EXIT_FAILURE);
  }
uint16_t bsize=*(uint16_t *)(block+OFFSET_BSIZE);
printf("Block size \t %d\n",bsize);
uint8_t csize=*(uint8_t *)(block+OFFSET_CSIZE);
printf("Cluster size \t %d\n",csize);
uint8_t rsize=*(uint8_t *)(block+OFFSET_RSIZE);
printf("Reserved size \t %d\n",rsize);
uint8_t nbfat=*(uint8_t *)(block+OFFSET_NBFAT);
printf("# of FAT \t %d\n",nbfat);
uint32_t fsize=*(uint16_t *)(block+OFFSET_FSIZE16);
if(fsize==0) fsize=*(uint32_t *)(block+OFFSET_FSIZE32);
printf("Size of FAT \t %d\n",fsize);
uint32_t total=*(uint16_t *)(block+OFFSET_COUNT16);
if(total==0) total=*(uint32_t *)(block+OFFSET_COUNT32);
printf("# of blocks \t %d\n",total);
uint16_t fspos=*(uint16_t *)(block+OFFSET_FSPOS);
printf("Index of FS \t %d\n",fspos);
int i;
for(i=0;i<fspos-1;i++){
  nb=read(b,block,BLOCK_SIZE);
  if(nb!=BLOCK_SIZE){
    fprintf(stderr,"Cannot read sector\n");
    exit(EXIT_FAILURE);
    }
  }
if(fspos<total){
  nb=read(b,block,BLOCK_SIZE);
  if(nb!=BLOCK_SIZE){
    fprintf(stderr,"Cannot read FS sector\n");
    exit(EXIT_FAILURE);
    }
  printf("Magic word \t %c %c %c %c\n",block[0],block[1],block[2],block[3]);
  uint32_t free=*(uint32_t *)(block+OFFSET_FREE);
  printf("Free clusters \t %d\n",free);
  uint32_t alloc=*(uint32_t *)(block+OFFSET_ALLOC);
  printf("Used clusters \t %d\n",alloc);
  printf("Allocated blocks %d/%d => %d%%\n",alloc*csize,total,alloc*csize*100/total);
  }
close(b);
return 0;
}
